@@ -1,9 +1,9 @@
 #!/usr/bin/env perl
-# $Id: fmtutil.pl 63934 2022-07-19 01:58:57Z preining $
+# $Id: fmtutil.pl 75097 2025-05-04 17:51:06Z karl $
 # fmtutil - utility to maintain format files.
 # (Maintained in TeX Live:Master/texmf-dist/scripts/texlive.)
 # 
-# Copyright 2014-2021 Norbert Preining
+# Copyright 2014-2025 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 #
@@ -24,11 +24,11 @@ BEGIN {
   TeX::Update->import();
 }
 
-my $svnid = '$Id: fmtutil.pl 63934 2022-07-19 01:58:57Z preining $';
-my $lastchdate = '$Date: 2022-07-19 03:58:57 +0200 (Tue, 19 Jul 2022) $';
+my $svnid = '$Id: fmtutil.pl 75097 2025-05-04 17:51:06Z karl $';
+my $lastchdate = '$Date: 2025-05-04 19:51:06 +0200 (Sun, 04 May 2025) $';
 $lastchdate =~ s/^\$Date:\s*//;
 $lastchdate =~ s/ \(.*$//;
-my $svnrev = '$Revision: 63934 $';
+my $svnrev = '$Revision: 75097 $';
 $svnrev =~ s/^\$Revision:\s*//;
 $svnrev =~ s/\s*\$$//;
 my $version = "r$svnrev ($lastchdate)";
@@ -42,9 +42,9 @@ use Cwd;
 # don't import anything automatically, this requires us to explicitly
 # call functions with TeXLive::TLUtils prefix, and makes it easier to
 # find and if necessary remove references to TLUtils
-use TeXLive::TLUtils qw();
+use TeXLive::TLUtils qw(wndws);
 
-require TeXLive::TLWinGoo if TeXLive::TLUtils::win32;
+require TeXLive::TLWinGoo if wndws();
 
 # numerical constants
 my $FMT_NOTSELECTED = 0;
@@ -53,8 +53,8 @@ my $FMT_FAILURE     = 2;
 my $FMT_SUCCESS     = 3;
 my $FMT_NOTAVAIL    = 4;
 
-my $nul = (win32() ? 'nul' : '/dev/null');
-my $sep = (win32() ? ';' : ':');
+my $nul = (wndws() ? 'nul' : '/dev/null');
+my $sep = (wndws() ? ';' : ':');
 
 my @deferred_stderr;
 my @deferred_stdout;
@@ -84,7 +84,7 @@ chomp(our $TEXMFSYSCONFIG = `kpsewhich -var-value=TEXMFSYSCONFIG`);
 chomp(our $TEXMFHOME = `kpsewhich -var-value=TEXMFHOME`);
 
 # make sure that on windows *everything* is in lower case for comparison
-if (win32()) {
+if (wndws()) {
   $TEXMFDIST = lc($TEXMFDIST);
   $TEXMFVAR = lc($TEXMFVAR);
   $TEXMFSYSVAR = lc($TEXMFSYSVAR);
@@ -280,7 +280,7 @@ sub main {
     # but for compatibility we'll silently keep the option.
     $cmd = 'edit';
     my $editor = $ENV{'VISUAL'} || $ENV{'EDITOR'};
-    $editor ||= (&win32 ? "notepad" : "vi");
+    $editor ||= (&wndws ? "notepad" : "vi");
     if (-r $changes_config_file) {
       &copyFile($changes_config_file, $bakFile);
     } else {
@@ -383,13 +383,18 @@ sub log_to_status {
 sub callback_build_formats {
   my ($what, $whatarg) = @_;
 
+  # sometimes (missing, all) there is no argument passed.
+  # Avoid warning from undef value being logged.
+  # https://tug.org/pipermail/tex-live/2023-September/049526.html
+  $whatarg = "" if ! defined $whatarg;
+
   # set up a tmp dir
   # On W32 it seems that File::Temp creates restrictive permissions (ok)
   # that are copied over with the files created inside it (not ok).
   # So make our own temp dir.
   my $tmpdir = "";
   if (! $opts{"dry-run"}) {
-    if (win32()) {
+    if (wndws()) {
       my $foo;
       my $tmp_deflt = File::Spec->tmpdir;
       for my $i (1..5) {
@@ -432,7 +437,12 @@ sub callback_build_formats {
   die "abs_path failed, strange: $!" if !$opts{'fmtdir'};
   print_info("writing formats under $opts{fmtdir}\n"); # report
 
-  # code taken over from the original shell script for KPSE_DOT etc
+  # Set KPSE_DOT in the environment so that kpathsea's expand functions
+  # will prepend it to relative paths when looking for files. This is
+  # needed because we chdir to a temp dir, but we want (say) "." as a
+  # path element to refer to the original working directory where the
+  # script was run, not the temp dir.
+  # 
   my $thisdir = cwd();
   $ENV{'KPSE_DOT'} = $thisdir;
   # due to KPSE_DOT, we don't search the current directory, so include
@@ -445,6 +455,9 @@ sub callback_build_formats {
   # for MFBASES.
   $ENV{'TEXFORMATS'} ||= "";
   $ENV{'TEXFORMATS'} = "$tmpdir$sep$ENV{TEXFORMATS}";
+  #
+  # ensure that LC_ALL is set to C to guarantee luatex generating formats
+  $ENV{'LC_ALL'} = "C";
 
   # switch to temporary directory for format generation; on the other hand,
   # for -n, the tmpdir won't exist, but we don't want to find a spurious
@@ -519,7 +532,7 @@ sub callback_build_formats {
   print_info("failed to build: $err (@err)\n")       if ($err);
   print_info("total formats: $total\n");
   chdir($thisdir) || warn "chdir($thisdir) failed: $!";
-  if (win32()) {
+  if (wndws()) {
     # try to remove the tmpdir with all files
     TeXLive::TLUtils::rmtree($tmpdir);
   }
@@ -709,6 +722,15 @@ sub rebuild_one_format {
   # get rid of leading * in inifiles
   $inifile =~ s/^\*//;
 
+  # Add -kanji-internal option for create (e-)p(La)TeX format
+  # with (e-)upTeX's pTeX compatible mode.
+  if ($eng =~ /^e?uptex$/
+      && $fmt =~ /^e?p/
+      && $addargs !~ /-kanji-internal=/) {
+    my $kanji = wndws() ? "sjis" : "euc";
+    $addargs = "-kanji-internal=$kanji " . $addargs;
+  }
+
   if ($fmt eq "metafun")       { $prgswitch .= "mpost"; }
   elsif ($fmt eq "mptopdf")    { $prgswitch .= "context"; }
   elsif ($fmt =~ m/^cont-..$/) { $prgswitch .= "context"; }
@@ -770,13 +792,37 @@ sub rebuild_one_format {
 
   #
   # check for existence of $engine
-  # we do *NOT* use the return value but rely on execution of the shell
+  # we do *NOT* use the return value but rely on execution of the shell.
+  # This won't find engines in "." (etc.) in PATH, because we have cd'd
+  # to a temporary directory.
+  # 
+  # Although we could change our which() fn to prepend KPSE_DOT (like
+  # kpathsea does), it won't help, because we won't be able to execute
+  # an engine "./$eng" (since we're in the temp dir). Prepending
+  # KPSE_DOT to the executable name seems too much. Instead, tell the
+  # user to have PATH include the absolute directory.
   if (!TeXLive::TLUtils::which($eng)) {
-    if ($opts{'no-error-if-no-engine'} &&
-        ",$opts{'no-error-if-no-engine'}," =~ m/,$eng,/) {
+    if ($opts{'no-error-if-no-engine'}
+        && ",$opts{'no-error-if-no-engine'}," =~ m/,$eng,/) {
       return $FMT_NOTAVAIL;
     } else {
       print_deferred_error("not building $fmt due to missing engine: $eng\n");
+      # could be irrelevant if PATH didn't contain ., and won't find
+      # other cases like the engine being in .., but it's just a help message,
+      # in response to
+      # https://tug.org/pipermail/tlbuild/2025q1/005685.html
+      if (-x "$ENV{KPSE_DOT}/$eng") {
+        print_deferred_error(<<END_ENGINE_IN_CWD);
+It seems your engine ($eng)
+  is in the original working directory
+  ($ENV{KPSE_DOT}).
+Since fmtutil works in a new temporary directory, the engine can't
+  be run from elsewhere. Sorry.
+To use an engine from an arbitrary directory, set your PATH to include
+  the absolute directory where the engine resides, instead of a relative
+  directory like ".".
+END_ENGINE_IN_CWD
+      }
       return $FMT_FAILURE;
     }
   }
@@ -1192,7 +1238,7 @@ sub determine_config_files {
           die "$prg: Config file \"$f\" not found";
         }
       }
-      push @tmp, (win32() ? lc($f) : $f);
+      push @tmp, (wndws() ? lc($f) : $f);
     }
     @{$opts{'cnffile'}} = @tmp;
     # in case that config files are given on the command line, the first
@@ -1203,12 +1249,12 @@ sub determine_config_files {
     chomp(@all_files);
     my @used_files;
     for my $f (@all_files) {
-      push @used_files, (win32() ? lc($f) : $f);
+      push @used_files, (wndws() ? lc($f) : $f);
     }
     #
     my $TEXMFLOCALVAR;
     my @TEXMFLOCAL;
-    if (win32()) {
+    if (wndws()) {
       chomp($TEXMFLOCALVAR =`kpsewhich --expand-path=\$TEXMFLOCAL`);
       @TEXMFLOCAL = map { lc } split(/;/ , $TEXMFLOCALVAR);
     } else {
@@ -1344,7 +1390,7 @@ sub save_fmtutil {
 #   and reset it to the real home dir of root.
 
 sub reset_root_home {
-  if (!win32() && ($> == 0)) {  # $> is effective uid
+  if (!wndws() && ($> == 0)) {  # $> is effective uid
     my $envhome = $ENV{'HOME'};
     # if $HOME isn't an existing directory, we don't care.
     if (defined($envhome) && (-d $envhome)) {
@@ -1413,16 +1459,6 @@ sub print_deferred_error {
 }
 
 
-# copied from TeXLive::TLUtils to reduce dependencies
-sub win32 {
-  if ($^O =~ /^MSWin/i) {
-    return 1;
-  } else {
-    return 0;
-  }
-}
-
-
 
 # version, help.
 
@@ -1479,7 +1515,7 @@ Options:
   --force                 (does nothing, exists for compatibility)
   --test                  (does nothing, exists for compatibility)
 
-Commands:
+Commands (exactly one must be specified):
   --all                   recreate all format files
   --missing               create all missing format files
   --byengine ENGINE       (re)create formats built with ENGINE
@@ -1607,6 +1643,12 @@ Supporting development binaries:
   easily running development binaries in parallel with the released
   binaries.
 
+Environment:
+
+  This script runs TeX and Metafont to generate the fmt/base file, and
+  thus all normal environment variables and search path rules for TeX/MF
+  apply.
+
 Report bugs to: tex-live\@tug.org
 TeX Live home page: <https://tug.org/texlive/>
 EOF
