@@ -1,20 +1,24 @@
 #!/usr/bin/env ruby
 # encoding: utf-8
 
+require 'byebug'
+
 # this file auto-generates tlpsrc files for hyphenation patterns - to be improved
 
-require_relative 'languages.rb'
-include Language::TeXLive
+require_relative 'lib/tex/hyphen'
+require_relative 'lib/tex/hyphen/texlive'
+include TeX::Hyphen
 include TeXLive
 
 #--------#
 # TLPSRC #
 #--------#
 
+print '[1;36mGenerating[0m [0;34m.tlpsrc files[0m for packages: '
 Package.all.sort.each do |package|
 	tlpsrcname = File.join(PATH::TLPSRC, sprintf('hyphen-%s.tlpsrc', package.name.safe))
 	file_tlpsrc = File.open(tlpsrcname, 'w')
-	printf "generating %s\n", tlpsrcname
+	print '[0;32m', package.name.safe, '[0m '
 
 	file_tlpsrc.puts "category TLCore"
 	package.list_dependencies.each do |dependency|
@@ -23,25 +27,15 @@ Package.all.sort.each do |package|
 
 	# FIXME Still doesn’t work well for Latin
 	file_tlpsrc.printf "shortdesc %s.\n", package.description_s
-	package.description_l.each do |line|
+	package.description.split("\n").each do |line|
 		file_tlpsrc.printf "longdesc %s\n", line
 	end
 	package.languages.each do |language|
-		if language.description_s && language.description_l then
-			# file_tlpsrc.printf "shortdesc %s.\n", language.description_s
-			# language.description_l.each do |line|
-			# 	file_tlpsrc.printf "longdesc %s\n", line
-			# end
-		end
-
-		file_tlpsrc.printf  "execute AddHyphen \\\n\tname=%s%s \\\n", language.name.safe, language.list_synonyms
+		file_tlpsrc.printf  "execute AddHyphen \\\n\tname=%s%s \\\n", language.babelname, language.list_synonyms
 		file_tlpsrc.printf "\t%s \\\n\t%s", language.list_hyphenmins, language.list_loader
 		if language.patterns_line + language.exceptions_line != ""
 			file_tlpsrc.printf " \\\n\t%s \\\n\t%s", language.patterns_line, language.exceptions_line
 		end
-		if language.code == "mn-cyrl-x-lmc" then
-			file_tlpsrc.printf " \\\n\tluaspecial=\"disabled:only for 8bit montex with lmc encoding\""
-		end
 		# end-of-line
 		file_tlpsrc.puts
 	end
@@ -60,22 +54,24 @@ Package.all.sort.each do |package|
 		file_tlpsrc.printf "runpattern f texmf-dist/%s\n", filename
 	end
 	file_tlpsrc.close
+  # puts File.read(tlpsrcname) if ['chinese', 'latin'].include?(package.name)
+  # byebug if ['chinese', 'latin'].include?(package.name)
 end
+puts
 
-#--------------#
-# language.dat #
-#--------------#
-language_dat_filename = File.join PATH::LANGUAGE_DAT, 'language.dat'
-File.open(language_dat_filename, 'w') do |file_language_dat|
-	printf "Generating %s\n", language_dat_filename
+#--------------------------#
+# language.dat and friends #
+#--------------------------#
+ldatfile = File.join PATH::LANGUAGE_DAT, 'language.dat'
+File.open(ldatfile, 'w') do |file|
+	puts '[1;36mGenerating[0m [0;34mlanguage.dat[0m'
 	Package.all.sort.each do |package|
 		package.languages.each do |language|
-			# main language name
-			file_language_dat.printf "%s\t%s\n", language.name.safe, language.loadhyph
-
-			# synonyms
+			# Main language name
+			file.printf "%s\t%s\n", language.babelname, language.loadhyph
+			# Synonyms
 			language.synonyms.each do |synonym|
-				file_language_dat.printf "=%s\n", synonym
+				file.printf "=%s\n", synonym
 			end
 		end
 	end
