@@ -73,8 +73,13 @@ module TeX
         end
 
         def list_run_files
+          # byebug if @bcp47 == "grc-x-ibycus"
           if use_old_loader
-            return [path('tex', "hyph-#{@bcp47}.tex")]
+            unless @bcp47 == "grc-x-ibycus" # FIXME AR 2025-03-31
+              return [path('tex', "hyph-#{@bcp47}.tex")]
+            else
+              return []
+            end
           else # We’re not returning here, but it makes the branching clearer
             files = [File.join(PATH::HYPHU8, 'loadhyph', loadhyph)]
           end
@@ -83,7 +88,8 @@ module TeX
             files << path('quote', sprintf("hyph-quote-%s.tex", bcp47))
           end
 
-          files << path('tex', sprintf('hyph-%s.tex', bcp47))
+          # byebug if @bcp47 =~ /^grc/
+          files << path('tex', sprintf('hyph-%s.tex', bcp47)) # unless @bcp47 == "grc-x-ibycus" # FIXME AR 2025-03-31
           Dir.glob(File.join('hyph-utf8', path('ptex', "hyph-#{bcp47}.*.tex"))) do |file|
             files << file.gsub(/^hyph-utf8\//, '')
           end
