@@ -1,6 +1,6 @@
 --
 -- polyglossia-cjk-spacing.lua
--- part of polyglossia v2.6 -- 2025/06/30
+-- part of polyglossia v2.8 -- 2025/09/22
 --
 
 local glyph_id = node.id"glyph"
@@ -10,6 +10,10 @@ local glue_id  = node.id"glue"
 local penalty_id = node.id"penalty"
 local whatsit_id = node.id"whatsit"
 local math_id  = node.id"math"
+local localpar_id = node.id"local_par"
+local ins_id = node.id"ins"
+local vadjust_id = node.id"adjust"
+local mark_id = node.id"mark"
 
 --
 -- attr_cjk: variant = plain: 0, JP/classic: 1, KR/modern: 2, SC: 3, TC: 4
@@ -373,7 +377,7 @@ local function insert_penalty_glue (head, curr, f, var, nobr, x)
     if nobr then
         local penalty = get_new_penalty(10000)
         head, curr = node.insert_after(head, curr, penalty)
-    elseif var == 0 or var == 2 then
+    elseif (var == 0 or var == 2) and curr.id ~= penalty_id then
         local penalty = get_new_penalty(50)
         head, curr = node.insert_after(head, curr, penalty)
     end
@@ -389,33 +393,67 @@ local function insert_penalty_glue (head, curr, f, var, nobr, x)
     return head, curr
 end
 
+--
+-- overwrite \inhibitglue which suppresses inter-character glue
+--
+local inhibitglue_attr  = luatexbase.new_attribute"polyglossia_inhibitglue_attr"
+local inhibitglue_index = luatexbase.new_luafunction"polyglossia_inhibitglue_func"
+lua.get_functions_table()[inhibitglue_index] = function ()
+    local n = get_new_glue(0)
+    node.set_attribute(n, inhibitglue_attr, 1)
+    node.write(n)
+end
+token.set_lua("inhibitglue", inhibitglue_index, "global", "protected")
+
 --
 -- main process for linebreak and inter-character spacing
 --
 local function cjk_break (head)
     local curr = head
+
     while curr do
-        if attr_cjk and (curr.id == glyph_id or curr.id == math_id and curr.subtype == 1) then
+        if curr.id == glyph_id
+            or curr.id == math_id and curr.subtype == 1 -- math_off
+            or curr.id == hbox_id and curr.subtype == 2 then -- normal box
+
             local var = node.has_attribute(curr, attr_cjk)
             if var then
-                local c, f = curr.char or 0, curr.font
+                local c, f
+                if curr.id == hbox_id then
+                    local t = curr.list and node.tail(curr.list)
+                    c, f = t and node.has_attribute(t, attr_cjk) and t.char, t and t.font
+                    if not c then goto skip_combining end
+                else
+                    c, f = curr.char or 0, curr.font
+                end
                 local cc, cjkc = get_charclass(var, c), is_cjk(c)
 
                 -- compress cjk punctuations when charclass is 1 thru 4
-                if var > 0 and cc > 0 and cc < 5 then
+                if curr.char and var > 0 and cc > 0 and cc < 5 then
                     head, curr = glyph_to_box(head, curr, cc)
                 end
 
                 local next = node.getnext(curr)
-                while next and next.id == whatsit_id do -- skip whatsit nodes
+                while next and (next.id == whatsit_id or next.id == penalty_id
+                    or next.id == hbox_id and next.next and next.next.id == ins_id
+                    or next.id == ins_id or next.id == vadjust_id or next.id == mark_id) do
+                    -- skip whatsit/penalty/footnote/ins/vadjust/mark nodes
                     curr, next = next, node.getnext(next)
                 end
+                if not next then break end
 
-                if next and node.has_attribute(next, attr_cjk)
-                    and (next.id == glyph_id or next.id == math_id and next.subtype == 0) then
+                if node.has_attribute(next, attr_cjk) and (next.id == glyph_id
+                    or next.id == math_id and next.subtype == 0 -- math_on
+                    or next.id == hbox_id and next.subtype == 2) then
 
-                    local n = next.char or 0
-                    f = f or next.font or 0 -- in case of curr == math_off
+                    local n, f2
+                    if next.id == hbox_id then
+                        local t = next.head
+                        n, f2 = t and node.has_attribute(t, attr_cjk) and t.char, t and t.font
+                        if not n then goto skip_combining end
+                    else
+                        n, f2 = next.char or 0, next.font
+                    end
 
                     -- skip combining. or dash+dash case to suppress stretching
                     if nobr_before[n] == 2 or (nobr_before[c] == 0 and nobr_before[n] == 0) then
@@ -423,6 +461,8 @@ local function cjk_break (head)
                     end
 
                     local nc = get_charclass(var, n)
+                    f = nc > 0 and f2 or f or f2 -- priority to cjk punctuation font
+                    if not f then goto skip_combining end
                     local nobr = nobr_before[n] or nobr_after[c]
 
                     -- insert spacing as of intercharclass
@@ -446,6 +486,33 @@ local function cjk_break (head)
                             end
                         end
                     end
+
+                elseif var > 0 and intercharclass[cc][0] then
+                    if next.id == glue_id and (next.subtype >= 13 and next.subtype <= 15
+                        or node.has_attribute(next, inhibitglue_attr)) then
+                        -- skip spaceskip, xspaceskkp, parfillskip, inhibitglue
+                    else
+                        -- insert glue between cjk_punct and non-glyph node
+                        head, curr = insert_cjk_penalty_glue(head, curr, f, var, cc, 0, false)
+                    end
+                end
+            end
+        elseif curr.id == localpar_id
+            or curr.id == hbox_id and curr.subtype == 3 -- indentbox
+            or curr.id == glue_id and (curr.subtype >= 13 and curr.subtype <= 15
+                                      or node.has_attribute(curr, inhibitglue_attr))
+            then
+            -- skip
+        else
+            local n = node.getnext(curr)
+            if n and n.id == glyph_id then
+                local var = node.has_attribute(n, attr_cjk)
+                if var and var > 0 then
+                    local nc = get_charclass(var, n.char)
+                    if intercharclass[0][nc] then
+                        -- insert glue between non-glyph node and cjk_punct
+                        head, curr = insert_cjk_penalty_glue(head, curr, n.font, var, 0, nc, false)
+                    end
                 end
             end
         end
