@@ -1,16 +1,7 @@
--- tkz_elements_parallelogram.lua
--- date 2025/06/15
--- version 4.15c
--- Copyright 2025  Alain Matthes
--- This work may be distributed and/or modified under the
--- conditions of the LaTeX Project Public License, either version 1.3
--- of this license or (at your option) any later version.
--- The latest version of this license is in
--- http://www.latex-project.org/lppl.txt
--- and version 1.3 or later is part of all distributions of LaTeX
--- version 2005/12/01 or later.
--- This work has the LPPL maintenance status “maintained”.
--- The Current Maintainer of this work is Alain Matthes.
+-- File: tkz_elements_parallelogram.lua
+-- Copyright (c) 2023–2025 Alain Matthes
+-- SPDX-License-Identifier: LPPL-1.3c
+-- Maintainer: Alain Matthes
 ---------------------------------------------------------------------------
 --                           Parallelogram
 ---------------------------------------------------------------------------
@@ -52,13 +43,13 @@ function parallelogram:new(za, zb, zc, zd)
 end
 
 setmetatable(parallelogram, {
-  __call = function(cls, ...)
-    return cls:new(...)
-  end
+	__call = function(cls, ...)
+		return cls:new(...)
+	end,
 })
 
 function parallelogram:get()
-  return self.pa, self.pb, self.pc, self.pd
+	return self.pa, self.pb, self.pc, self.pd
 end
 
 function parallelogram:fourth(za, zb, zc)
