@@ -3,16 +3,24 @@
 -- SPDX-License-Identifier: LPPL-1.3c
 -- Maintainer: Alain Matthes
 
-
 -- tkz-settings and math
 
-tkz = {}
-tkz.nb_dec = 10
+tkz = tkz or {}
+tkz.nb_dec = 8
 tkz.epsilon = 0.1 ^ tkz.nb_dec
 tkz.dc = 2
 tkz.phi = (1 + math.sqrt(5)) / 2 -- golden number φ
 tkz.invphi = (math.sqrt(5) - 1) / 2
 tkz.sqrtphi = math.sqrt(tkz.phi)
+tkz.rad = 180 / math.pi
+tkz.deg = math.pi / 180
+tkz.pt = 254 / 7227
+tkz.tau = 2 * math.pi
+-- real
+function tkz.approx(x, y, EPS)
+	 EPS = EPS or tkz.epsilon
+	return math.abs(x - y) <= EPS
+end
 
 function tkz.midpoint(a, b)
 	return (a + b) / 2
@@ -51,20 +59,31 @@ end
 -------------- angles -------------------
 -----------------------------------------
 function tkz.get_angle(a, b, c)
-	return angle_normalize_(get_angle_(a, b, c))
+	return get_angle_(a, b, c)
 end
 
-function tkz.angle_normalize(a)
-	return angle_normalize_(a)
+function tkz.inner_angle(pa, pb, pc)
+	return inner_angle_(pa, pb, pc)
+end
+
+function tkz.angle_normalize(an)
+	return angle_normalize_(an)
 end
 
 function tkz.is_direct(pa, pb, pc)
 	return get_angle_(pa, pb, pc) > 0
 end
 
+function tkz.get_angle_normalize(a, b, c)
+	return get_angle_normalize_(a, b, c)
+end
+
+function tkz.angle_between_vectors(a, b, c, d)
+	return tkz_angle_between_vectors_(a, b, c, d)
+end
+
 function tkz.round(num, idp)
-	idp = idp or 0
-	local mult = 10 ^ idp
+	local mult = 10 ^ (idp or 0)
 	return math.floor(num * mult + 0.5) / mult
 end
 
@@ -240,8 +259,8 @@ function tkz.solve_cubic_(a, b, c, d)
 
 		local theta = math.acos(t)
 		local x1 = 2 * cubert(r) * math.cos(theta / 3) - b / (3 * a)
-		local x2 = 2 * cubert(r) * math.cos((theta + 2 * math.pi) / 3) - b / (3 * a)
-		local x3 = 2 * cubert(r) * math.cos((theta + 4 * math.pi) / 3) - b / (3 * a)
+		local x2 = 2 * cubert(r) * math.cos((theta + tkz.tau) / 3) - b / (3 * a)
+		local x3 = 2 * cubert(r) * math.cos((theta + 2 * tkz.tau) / 3) - b / (3 * a)
 
 		return x1, x2, x3
 	end
@@ -263,6 +282,7 @@ end
 function tkz.is_linear(z1, z2, z3)
 	return math.abs((z2 - z1) ^ (z3 - z1)) < tkz.epsilon
 end
+tkz.aligned = tkz.is_linear
 
 function tkz.is_ortho(z1, z2, z3)
 	return math.abs((z2 - z1) .. (z3 - z1)) < tkz.epsilon
@@ -280,6 +300,21 @@ function tkz.residue(x)
 	return residue_(x)
 end
 
+-- internal utility: test for NaN
+function tkz.is_NaN(x)
+	return x ~= x
+end
+
+-- internal utility: test for Infinity
+function tkz.is_Inf(x)
+	return (x == math.huge) or (x == -math.huge)
+end
+
+-- internal utility: detect invalid numeric results (NaN or ±Inf)
+function tkz.notDef(x)
+	return tkz.is_NaN(x) or tkz.is_Inf(x)
+end
+
 function tkz.is_integer(x)
 	return type(x) == "number"
 		and x == x -- exclude NaN
@@ -322,7 +357,6 @@ function tkz.display(z)
 	return tkz_display_(z)
 end
 
-
 function tkz.poncelet_point(a, b, c, d)
 	local e1 = euler_center_(a, b, c)
 	local e2 = euler_center_(a, c, d)
@@ -340,14 +374,149 @@ function tkz.poncelet_point(a, b, c, d)
 	end
 end
 
-
 function tkz.orthopole(a, b, c, l)
 	local ap, bp, cp = l:projection(a, b, c)
 	local bpp = self.ca:projection(bp)
 	local app = self.bc:projection(ap)
-	local la = line(ap, app)
-	local lb = line(bp, bpp)
+	local la = line:new(ap, app)
+	local lb = line:new(bp, bpp)
 	return intersection(la, lb)
 end
 
+function tkz.nodes_from_paths(PAcenters, PAthrough, wbase, tbase, indice)
+	wbase  = wbase  or "w"
+	tbase  = tbase  or "t"
+	local n = #PAcenters
+	local k0 = (indice or 1) - 1   -- offset d’indice
+
+	for k = 1, n do
+		local i = k0 + k
+		z[wbase .. i] = PAcenters:get(k)
+		z[tbase .. i] = PAthrough:get(k)
+	end
+end
+
+-- tkz.fsolve
+-- Cherche des racines de f sur [a,b] en découpant l'intervalle en n sous-intervales
+-- et en raffinant par une itération de type Newton avec dérivée numérique.
+-- Arguments :
+--   f : function(x) -> number
+--   a, b : bornes (numbers)
+--   n : nombre de sous-intervalles (par défaut 25)
+--   opts (optionnel, table) :
+--       tol       : tolérance sur |f(r)| (déf. 1e-8)
+--       step_tol  : tolérance sur le déplacement de Newton (déf. 1e-10)
+--       h         : pas pour la dérivée finie (déf. 1e-6)
+--       max_iter  : itérations max par graine (déf. 8)
+--       merge_eps : distance pour fusionner deux racines proches (déf. 1e-5)
+function tkz.fsolve(f, a, b, n, opts)
+	if type(f) ~= "function" or type(a) ~= "number" or type(b) ~= "number" then return end
+	if a == b then return end
+	if a > b then a, b = b, a end
+
+	n = n or 25
+	opts = opts or {}
+	local tol       = opts.tol       or 1e-8
+	local step_tol  = opts.step_tol  or 1e-10
+	local h         = opts.h         or 1e-6
+	local max_iter  = opts.max_iter  or 8
+	local merge_eps = opts.merge_eps or 1e-5
+
+	local function isfinite(x)
+		return (x == x) and (x ~= math.huge) and (x ~= -math.huge)
+	end
+
+	local function safe_f(x)
+		local ok, y = pcall(f, x)
+		if not ok or not isfinite(y) then return nil end
+		return y
+	end
+
+	local function push_unique(S, r)
+		for i = 1, #S do
+			if math.abs(S[i] - r) <= merge_eps then
+				-- remplace par la moyenne pour stabiliser
+				S[i] = 0.5 * (S[i] + r)
+				return
+			end
+		end
+		S[#S+1] = r
+	end
+
+	local S = {}
+	local delta = (b - a) / n
+	local x = a
+	local fin = x + delta
+
+	while x < b do
+		-- Graine : milieu de [x, fin]
+		local r = 0.5 * (x + fin)
+
+		-- Optionnel : si f(x) et f(fin) existent et n'encadrent rien,
+		-- on tente quand même (peut attraper un zéro au sommet).
+		local fr = safe_f(r)
+		if fr ~= nil then
+			for _ = 1, max_iter do
+				local frh = safe_f(r + h)
+				if frh == nil then break end
+				local denom = frh - fr
+				if denom == 0 then break end
+
+				local step = fr * h / denom
+				if not isfinite(step) then break end
+
+				local r_next = r - step
+				if not isfinite(r_next) then break end
+
+				-- Arrêt sur petit déplacement
+				if math.abs(step) <= step_tol * (1 + math.abs(r)) then
+					r = r_next
+					break
+				end
+
+				r = r_next
+				fr = safe_f(r)
+				if fr == nil then break end
+				-- Arrêt sur résidu
+				if math.abs(fr) <= tol then break end
+			end
+
+			-- Valide si : dans [x, fin) et résidu petit
+			local fr_ok = fr ~= nil and math.abs(fr) <= tol
+			if fr_ok and (r >= x) and (r < fin) then
+				push_unique(S, r)
+			end
+		end
+
+		x = fin
+		fin = x + delta
+	end
+
+	if #S > 0 then return S end
+end
+
+-- tkz.derivative
+-- Compute the numerical derivative f'(x0)
+-- using the central difference formula.
+-- Optional parameter 'accuracy' controls precision (default = 1e-6).
+function tkz.derivative(f, x0, accuracy)
+	if type(f) ~= "function" or type(x0) ~= "number" then return nil end
+	accuracy = accuracy or 1e-6
+	local f1, f2 = f(x0 + accuracy), f(x0 - accuracy)
+	if (f1 == nil) or (f2 == nil) then return nil end
+	return (f1 - f2) / (2 * accuracy)
+end
+
+function tkz.range(a, b, step)
+	step = step or 1
+	if step == 0 then error("step must not be zero") end
+	local tbl = {}
+	if (a <= b and step > 0) or (a >= b and step < 0) then
+		for k = a, b, step do
+			table.insert(tbl, k)
+		end
+	end
+	return tbl
+end
+
 return tkz
