@@ -5,7 +5,41 @@
 
 triangle = {}
 triangle.__index = triangle
+
+
 function triangle:new(za, zb, zc)
+	local eps = tkz.epsilon
+	-- ===== Guardrails against degeneration =====
+	-- sall vertices combined
+	if point.abs(za - zb) <= eps or point.abs(zb - zc) <= eps or point.abs(zc - za) <= eps then
+		tex.error("Degenerate triangle: two vertices coincide.")
+		return
+	end
+	-- aligned vertices
+	if tkz.is_linear(za, zb, zc) then
+		tex.error("Degenerate triangle: vertices aligned.")
+		return
+	end
+
+	-- ===== Side lengths =====
+	local c = point.abs(zb - za)
+	local a = point.abs(zc - zb)
+	local b = point.abs(za - zc)
+
+	-- ===== Line objects =====
+	local ab = line:new(za, zb)
+	local bc = line:new(zb, zc)
+	local ca = line:new(zc, za)
+
+	-- ===== Angles =====
+	local alpha = math.abs(get_angle_(za, zb, zc))
+	local beta = math.abs(get_angle_(zb, zc, za))
+	local gamma = math.abs(get_angle_(zc, za, zb))
+
+	local alpha_ = angle:new(za, zb, zc)
+	local beta_ = angle:new(zb, zc, za)
+	local gamma_ = angle:new(zc, za, zb)
+
 	local type = "triangle"
 	local circumcenter = circum_center_(za, zb, zc)
 	local centroid = barycenter_({ za, 1 }, { zb, 1 }, { zc, 1 })
@@ -14,19 +48,13 @@ function triangle:new(za, zb, zc)
 	local eulercenter = euler_center_(za, zb, zc)
 	local spiekercenter = spieker_center_(za, zb, zc)
 	local orientation = find_orientation_(za, zb, zc)
-	local c = point.abs(zb - za)
-	local a = point.abs(zc - zb)
-	local b = point.abs(za - zc)
-	local alpha = find_angle_(za, zb, zc)
-	local beta = find_angle_(zb, zc, za)
-	local gamma = find_angle_(zc, za, zb)
-	local ab = line(za, zb)
-	local ca = line(zc, za)
-	local bc = line(zb, zc)
+
 	local semiperimeter = (a + b + c) / 2
-	local area = math.sqrt(semiperimeter * (semiperimeter - a) * (semiperimeter - b) * (semiperimeter - c))
+	local cross = (zb - za) ^ (zc - za)
+	local area = 0.5 * math.abs((zb - za) ^ (zc - za))
 	local inradius = area / semiperimeter
 	local circumradius = (a * b * c) / (4 * area)
+
 	local tr = {
 		pa = za,
 		pb = zb,
@@ -47,9 +75,9 @@ function triangle:new(za, zb, zc)
 		alpha = alpha,
 		beta = beta,
 		gamma = gamma,
-		angle_alpha = angle_alpha,
-		angle_beta = angle_beta,
-		angle_gamma = angle_gamma,
+		alpha_ = alpha_,
+		beta_ = beta_,
+		gamma_ = gamma_,
 		semiperimeter = semiperimeter,
 		area = area,
 		inradius = inradius,
@@ -80,27 +108,38 @@ end
 -----------------------
 -- Result -> boolean
 -----------------------
-
-function triangle:in_out(pt)
-	return in_out_(self.pa, self.pb, self.pc, pt)
+-- added strict
+function triangle:in_out(pt, strict)
+	-- ne change pas → tous les anciens documents restent valides
+	return in_out_(self.pa, self.pb, self.pc, pt, strict)
 end
 
-function triangle:check_equilateral()
-	return check_equilateral_(self.pa, self.pb, self.pc)
+function triangle:check_equilateral(EPS)
+	return check_equilateral_(self.pa, self.pb, self.pc, EPS)
 end
 triangle.is_equilateral = triangle.check_equilateral
 
-function triangle:check_acutangle()
-	local asq = self.a * self.a
-	local bsq = self.b * self.b
-	local csq = self.c * self.c
-	if asq + bsq > csq and bsq + csq > asq and csq + asq > bsq then
-		return true
-	else
-		return false
-	end
+function triangle:check_acutangle(EPS)
+return check_acutangle_(self.pa, self.pb, self.pc, EPS)
 end
 triangle.is_acute = triangle.check_acutangle
+
+function triangle:check_isosceles(EPS)
+return check_isosceles_(self.pa, self.pb, self.pc, EPS)
+
+		-- if math.abs(a - b) <= eps then
+		-- 		return true, "c"      -- sommet C au-dessus de AB
+		-- elseif math.abs(b - c) <= eps then
+		-- 		return true, "a"      -- sommet A au-dessus de BC
+		-- elseif math.abs(c - a) <= eps then
+		-- 		return true, "b"      -- sommet B au-dessus de CA
+		-- else
+		-- 		return false
+		-- end
+end
+
+triangle.is_isosceles = triangle.check_isosceles
+
 -----------------------
 -- Result -> real
 -----------------------
@@ -109,15 +148,16 @@ function triangle:area() -- obsolete
 	return area_(self.pa, self.pb, self.pc)
 end
 
+
 function triangle:barycentric_coordinates(pt)
 	local xA, yA = self.pa:get()
 	local xB, yB = self.pb:get()
 	local xC, yC = self.pc:get()
 	local xP, yP = pt:get()
-	local orient2d_ABC = orient2d_(xA, yA, xB, yB, xC, yC)
-	local l_A = orient2d_(xP, yP, xB, yB, xC, yC) / orient2d_ABC
-	local l_B = orient2d_(xA, yA, xP, yP, xC, yC) / orient2d_ABC
-	local l_C = orient2d_(xA, yA, xB, yB, xP, yP) / orient2d_ABC
+	local orient2d_ABC = oriented_area2_(xA, yA, xB, yB, xC, yC)
+	local l_A = oriented_area2_(xP, yP, xB, yB, xC, yC) / orient2d_ABC
+	local l_B = oriented_area2_(xA, yA, xP, yP, xC, yC) / orient2d_ABC
+	local l_C = oriented_area2_(xA, yA, xB, yB, xP, yP) / orient2d_ABC
 	return l_A, l_B, l_C
 end
 
@@ -517,10 +557,77 @@ function triangle:orthopole(l)
 	local ap, bp, cp = l:projection(self.pa, self.pb, self.pc)
 	local bpp = self.ca:projection(bp)
 	local app = self.bc:projection(ap)
-	local la = line(ap, app)
-	local lb = line(bp, bpp)
+	local la = line:new(ap, app)
+	local lb = line:new(bp, bpp)
 	return intersection(la, lb)
 end
+
+
+function triangle:isodynamic_points()
+	-- Cas particulier : triangle équilatéral
+	if self:is_equilateral() then
+		-- On prend le centre de gravité G = (A + B + C)/3
+		local A, B, C = self.pa, self.pb, self.pc
+		local G = (A + B + C) / 3
+		-- On renvoie deux fois le même point pour garder la même signature
+		return G, G
+	end
+
+	-- Cas général
+	local a, b, c = self.a, self.b, self.c
+	local ab, bc = self.ab, self.bc, self.ca  -- self.ca est inutile ici mais cohérent
+
+	-- Cercles d’Apollonius associés aux rapports b/a sur AB et c/b sur BC
+	local Cab = ab:apollonius(b / a)
+	local Cbc = bc:apollonius(c / b)
+
+	-- L’intersection donne les deux points isodynamiques
+	return intersection(Cab, Cbc)
+end
+
+triangle.neuberg_points = triangle.isodynamic_points
+
+-- Points d’intersection du cercle d’Apollonius (b/a, côté AB)
+-- avec la droite (AB), en mettant en premier le point sur le segment [AB].
+
+function triangle:apollonius_points(side, EPS)
+	EPS = EPS or tkz.epsilon
+
+	local a, b, c = self.a, self.b, self.c  -- a = BC, b = CA
+	local A, B, C = self.pa, self.pb, self.pc
+  local ab, bc, ca = self.ab, self.bc, self.ca
+   local x, y, X, Y
+		if side == "ab"  then
+			L = self.ab
+			k = b / a
+			x, y = a, b
+			X, Y = A, B     -- AC / BC
+	elseif side == "bc"  then
+			L = self.bc
+			k = c / b
+			x, y = b, c
+			X, Y = B, C         -- AB / CA
+	elseif side == "ca"  then
+			L = self.ca
+			k = a / c
+			x, y = c, a
+			X, Y = C, A           -- BC / AB
+	else
+			tex.error("triangle:apollonius_circle -> invalid side: use 'ab', 'bc', or 'ca'")
+			return
+	end
+
+		if math.abs(x - y) <= EPS then
+		local M = (X + Y) / 2
+		return M, M
+	end
+
+     return L:harmonic(k)
+end
+
+-- Ancien nom si tu veux conserver la compatibilité :
+triangle.bisector_points = triangle.apollonius_points
+
 -------------------
 -- Result -> line
 -------------------
@@ -693,12 +800,60 @@ end
 -----------------------
 --- Result -> circles --
 -----------------------
+
+function triangle:apollonius_circle(side, EPS)
+		 EPS = EPS or tkz.epsilon
+		-- lengths
+		local a, b, c = self.a, self.b, self.c
+		local L, k
+
+		if side == "ab"  then
+				L = self.ab
+				k = b / a        -- AC / BC
+		elseif side == "bc"  then
+				L = self.bc
+				k = c / b        -- AB / CA
+		elseif side == "ca"  then
+				L = self.ca
+				k = a / c        -- BC / AB
+		else
+				tex.error("triangle:apollonius_circle -> invalid side: use 'ab', 'bc', or 'ca'")
+				return
+		end
+
+		-- cas k ≈ 1 : volontairement traité
+		if math.abs(k - 1) <= EPS then
+    	return L:bisector()
+		else
+		  return L:apollonius(k)
+		end
+end
+
+
+
+function triangle:three_apollonius_circles()
+	local a, b, c = self.a, self.b, self.c
+	local ab, bc, ca = self.ab, self.bc, self.ca
+	local Cab = ab:apollonius(b / a)
+	local Cbc = bc:apollonius(c / b)
+	local Cca = ca:apollonius(a / c)
+	return Cab, Cbc, Cca
+end
+
+
+function triangle:three_tangent_circles()
+	local i = self.incenter
+	local a, b, c = self:projection(i)
+	return circle:new(self.pa, c), circle:new(self.pb, a), circle:new(self.pc, b)
+end
+
+
 function triangle:euler_circle()
 	return circle:new(euler_center_(self.pa, self.pb, self.pc), midpoint_(self.pb, self.pc))
 end
 
 function triangle:circum_circle()
-	return circle:new(circum_circle_(self.pa, self.pb, self.pc), self.pa)
+	return circle:new(circum_center_(self.pa, self.pb, self.pc),self.pa)
 end
 
 function triangle:in_circle()
@@ -731,14 +886,14 @@ end
 
 function triangle:cevian_circle(p)
 	local pta, ptb, ptc = cevian_(self.pa, self.pb, self.pc, p)
-	return circle:new(circum_circle_(pta, ptb, ptc), pta)
+	return circle:new(circum_center_(pta, ptb, ptc), pta)
 end
 
 function triangle:symmedial_circle()
 	local pta, ptb, ptc, p
 	p = lemoine_point_(self.pa, self.pb, self.pc)
 	pta, ptb, ptc = cevian_(self.pa, self.pb, self.pc, p)
-	return circle:new(circum_circle_(pta, ptb, ptc), pta)
+	return circle:new(circum_center_(pta, ptb, ptc), pta)
 end
 
 function triangle:conway_circle()
@@ -786,7 +941,7 @@ function triangle:taylor_circle()
 	local d = projection_(self.pa, self.pb, a)
 	local e = projection_(self.pb, self.pc, b)
 	local f = projection_(self.pc, self.pa, c)
-	return circle:new(circum_circle_(d, e, f), d)
+	return circle:new(circum_center_(d, e, f), d)
 end
 
 function triangle:adams_circle()
@@ -839,41 +994,6 @@ function triangle:kenmotu_circle()
 	return circle:new(through(center, rho))
 end
 
--- Circle tangent to two straight lines passing through a given point
-function triangle:c_ll_p(p)
-	-- Compute the bisector of the triangle
-	local lbi = bisector(self.pa, self.pb, self.pc)
-
-	if lbi:in_out(p) then
-		-- Orthogonal projection of p onto the bisector
-		local lp = lbi:ortho_from(p)
-
-		-- Intersection of line from p to its projection with self.pa and self.pb
-		local i = intersection_ll_(p, lp.pb, self.pa, self.pb)
-
-		-- Intersection points of the line with the circle defined by (i, p)
-		local t1, t2 = intersection_lc_(self.pa, self.pb, i, p)
-
-		-- Create the main line and find orthogonal projections from t1 and t2
-		local lab = line:new(self.pa, self.pb)
-		local x = lab:ortho_from(t1).pb
-		local y = lab:ortho_from(t2).pb
-
-		-- Return two circles based on the orthogonal projections and points t1, t2
-		return circle:new(intersection_ll_(x, t1, self.pa, p), t1), circle:new(intersection_ll_(y, t2, self.pa, p), t2)
-	else
-		local lab = line:new(self.pa, self.pb)
-		-- Reflection of p across the bisector
-		local q = lbi:reflection(p)
-
-		-- Compute circles from the Wallis construction
-		local c1, c2 = lab:c_l_pp(p, q)
-
-		-- Return two circles with centers and points on their circumference
-		return c1, c2
-	end
-end
-
 
 -- Circle tangent to the sides originating from p and to the circle passing through the other two vertices.
 -- argument the vertex of the chosen triangle, the center of the circle passing through the other two vertices.
@@ -889,25 +1009,89 @@ function triangle:c_c(p, C)
 		a, b, c = self.pc, self.pa, self.pb
 	end
 	local Lm = self:mediator(p)
-	local n = intersection(Lm, C)
+	local n = intersection(Lm, C, {near = p})
 	local i = self.incenter
-	local m = line(c, i):reflection(n)
-	local L = line(c, m):parallel_from(i)
-	local Lac = line(a, c)
+	local q = line:new(c, i):reflection(n)
+	local L = line:new(c, q):parallel_from(i)
+	local Lac = line:new(a, c)
 	local e = intersection(L, Lac)
-	local Lia = line(i, a)
+	local Lia = line:new(i, a)
 	local Le = Lac:orthogonal_from(e)
 	return circle(intersection(Lia, Le), e)
 end
 triangle.thebault = triangle.c_c
 
-function triangle:three_tangent_circles()
-	local i = self.incenter
-	local a, b, c = self:projection(i)
-	return circle:new(self.pa, c), circle:new(self.pb, a), circle:new(self.pc, b)
+function triangle:thebault_circles(p)
+	local pa, pb, pc = self.pa, self.pb, self.pc
+	local a, b, c    -- sommets réordonnés
+	local side       -- côté sur lequel se trouve p (objet line)
+
+	--------------------------------------------------
+	-- 1. Déterminer le côté contenant p et (a,b,c)
+	--    a = sommet opposé au côté portant p
+	--------------------------------------------------
+	if self.bc:in_out_segment(p) then
+		-- p sur BC -> sommet opposé A
+		a, b, c = pa, pb, pc
+		side    = self.bc
+
+	elseif self.ca:in_out_segment(p) then
+		-- p sur CA -> sommet opposé B
+		a, b, c = pb, pc, pa
+		side    = self.ca
+
+	elseif self.ab:in_out_segment(p) then
+		-- p sur AB -> sommet opposé C
+		a, b, c = pc, pa, pb
+		side    = self.ab
+
+	else
+		tex.error("thebault_circles: point is not on any side of the triangle.")
+		return
+	end
+
+	--------------------------------------------------
+	-- 2. Construction des centres (comme dans ta version)
+	--------------------------------------------------
+	local i  = self.incenter        -- incenter du triangle ABC
+	local ic = in_center_(p, a, c)  -- incenter du triangle (p,a,c)
+	local ib = in_center_(p, a, b)  -- incenter du triangle (p,a,b)
+
+	-- projections de p sur (ic i) et (ib i)
+	local pc_ = projection_(p, ic, i)
+	local pb_ = projection_(p, ib, i)
+
+	-- intersections avec (b c)
+	local tc = intersection_ll_(i, pc_, b, c)
+	local tb = intersection_ll_(i, pb_, b, c)
+
+	-- perpendiculaires à (b c) par tc et tb
+	local xc = ortho_from_(tc, b, c)
+	local xb = ortho_from_(tb, b, c)
+
+	-- centres des deux cercles
+	local center_c = intersection_ll_(p, ic, tc, xc)
+	local center_b = intersection_ll_(p, ib, tb, xb)
+
+	if not center_b or not center_c then
+		tex.error("thebault_circles: failed to construct circle centers.")
+		return
+	end
+
+	--------------------------------------------------
+	-- 3. Points de tangence sur le côté et cercles
+	--------------------------------------------------
+	-- side:projection(pt1, pt2) renvoie les projections
+	local tb_tan, tc_tan = side:projection(center_b, center_c)
+
+	local Cb = circle:new(center_b, tb_tan)
+	local Cc = circle:new(center_c, tc_tan)
+
+	return Cb, Cc
 end
 
 
+
 function triangle:mixtilinear_incircle(arg)
 	local i = resolve_triangle_index(self, arg)
 
@@ -934,6 +1118,8 @@ function triangle:mixtilinear_incircle(arg)
 	return circle(W, P1)
 end
 
+
+
 -------------------
 -- Result -> triangle
 -------------------
@@ -1125,7 +1311,7 @@ function triangle:kiepert_hyperbola()
 	-- simson lines
 	local Lsa = self:simson_line(M)
 	local Lsb = self:simson_line(N)
-	local axis = bisector(center, Lsa.pa, Lsb.pa)
+	local axis = tkz.bisector(center, Lsa.pa, Lsb.pa)
 	-- new frame system
 	local minor = axis:ortho_from(center)
 	local sys = occs:new(minor, center)
@@ -1250,4 +1436,32 @@ function triangle:path(p1, p2)
 	return path:new(list)
 end
 
+--- CONTACT PROBLEM APOLLONIUS
+
+-- Circle tangent to the three lines of the triangle:
+-- returns (in this order): incircle, excircle at pa, excircle at pb, excircle at pc.
+-- Optionnel: which ∈ { nil/"all", "in", "pa", "pb", "pc", "ex_a", "ex_b", "ex_c" }
+function triangle:LLL(which)
+	which = which or "all"
+	-- --- sélecteur
+	local C_in = self:in_circle()
+	local C_ea = self:ex_circle(0) -- pa
+	local C_eb = self:ex_circle(1) -- pb
+	local C_ec = self:ex_circle(2) -- pc
+	if which == "all" or which == nil then
+		return C_in, C_ea, C_eb, C_ec
+	end
+
+	-- alias pratiques
+	if which == "in" then return C_in end
+	if which == "pa" or which == "ex_a" then return C_ea end
+	if which == "pb" or which == "ex_b" then return C_eb end
+	if which == "pc" or which == "ex_c" then return C_ec end
+
+	tex.error("Unknown selector for triangle:c_lll: "..tostring(which))
+
+end
+triangle.c_lll = triangle.LLL
+
+
 return triangle
